&HTML_MISC_HEADER("Developer's Guide");

print CLIENT <<EOF

<P><BR>

<A NAME="basics"><H2>The Basics</H2>

Adding support for a new package can be very simple, or very complex,
depending on your needs and the nature of the tool. Two libraries,
discussed in the next section, provide a set of functions to make this
development easier. <P>

Each package is made up of one or more tools. Each package as well as
each tool is refered to by a three letter code: the spi package
includes bat, psi, cdt, and cql; the cop package includes just cop.
The package code must be unique among all packages, and the tool code
must be unique among all tools, since they are used as identifiers
throughout Merlin. <P>

As far as Merlin is concerned, only three files are required for a new
tool: <CODE>merlin_config</CODE>, <CODE>execute.pl</CODE>, and
<CODE>read_report.pl.</CODE> All other file name are up to the
developer, although they must end in <CODE>.pl</CODE> if they are
written in Perl. Below is a closer look at the files required:

<H3>merlin_config</H3>

This is a data file which contains the name of the package, the code
for the package, and where the package can be found. It also contains
one or more tool descriptions (and their corresponding codes). The
formatting is as follows: <P>

<CODE>
[package code]:[directory]:[package title]<BR>
[tool1 code]:[tool1 title]<BR>
[tool2 code]:[tool2 title]<P>
</CODE>

The <CODE>merlin_config</CODE> file for SPI looks like this: <P>

<CODE>spi::Security Profile Inspector<BR>
bat:Binary Authentication Tool<BR>
qsp:Quick System Profile<BR>
psi:Password Security Inspector<BR>
cdt:Change Detector Tool<P>
</CODE>

Note that the directory location is blank; it will be filled in by the
Configuration link. <P>

<H3>execute.pl</H3>

This is the link to be offered from Merlin's Table of Contents. In the
case of TAMU Tiger, a form is offered, where the user can select which
checks to perform. For SPI, a sub-menu of tools offered by the
packages is provided.

<H3>read_report.pl</H3>

When a report is selected from the Report Browser, this program is
executed. It should read the report, and return to the CLIENT the
HTML-enhanced version. How this is done depends on the tool SPI, for
example, uses the <CODE>rg</CODE> program included with SPI, but uses
a different template file. The other tools actually parse the report
file. The argument passed in $html_script_args is the file name to
display (see <A HREF="server.html.pl">The HTTP Server</A> for more
details on argument passing).

<A NAME="library"><H2>Tools Library</H2>

The following functions are offered to all packages, in the file
<CODE>perl/tools.pl</CODE>:

<UL> <LI> <B><CODE>NTOM(<EM>num</EM>)</B></CODE>: Convert a numeric
value between 1 and 12 to its month equivalent. <P>

<LI> <B><CODE>TOOL_PARSE_FILE(<EM>filename</EM>)</B></CODE>: Take as
input a valid file and return an array containing the date, the time,
the host and the tool. <P>

The filename is formatted as follows:
<PRE>
[tool].[date].[time].[host]
</PRE>

where<P>

[tool] is a three character description of the tool executed. It
should be defined in the merlin_config file for that tool. <EM>Note
that this three character description should be unique among all
packages.</EM><P>

[date] is a 8 digit date, such as 19950713 for July 13, 1995.<P>

[time] is a 6 digit time, such as 165150 for 4:51 pm and 50 seconds.<P>

[host] is an arbitarly long host name (for now, always the local host).<P>


<LI> <B><CODE>TOOL_CREATE_FILE(<EM>tool</EM>)</B></CODE>: Create a
tool filename, formatted as described for TOOL_PARSE_FILE(). It takes
as a parameter a three character tool descriptor. <P>

<LI> <B><CODE>TOOL_RUN_STATUS(<EM>filename, pgid</EM>)</B></CODE>:
Take as input a process group ID, followed by filename (as created
with TOOL_CREATE_FILE()). It then writes the process group ID into the
status directory, under that filename. <P>

</UL>

Output to the HTTP Client should be sent to the file descriptor
CLIENT. To simplify the generation of HTML code, the following
functions are provided, in the file <CODE>perl/html.pl</CODE>:

<UL>

<LI> <B><CODE>HTML_MISC_HEADER(<EM>title</EM>)</B></CODE>: Sends the
standard Merlin HTML page header. This function is used by virtually
all pages, excluding the home page and the reports. <P>


<LI> <B><CODE>HTML_MISC_FOOTER()</B></CODE>: Sends the standard Merlin
HTML page footer. This function is used by virtually all pages,
excluding the home page and the reports. <P>


<LI> <B><CODE>HTML_REPORT_HEADER(<EM>file, title</EM>)</B></CODE>:
Sends the Merlin HTML page header used for reports. <P>

<LI> <B><CODE>HTML_RADIOS(<EM>variable, value, descript1,
descript2,...</EM>)</B></CODE>: Sends a set of radio buttons. The
first parameter is the name of the variable to fill (sent as a
string). The second parameter is the default value. Each subsequent
parameter is a description for each button in the set. <P>

<LI> <B><CODE>HTML_CHECKBOXES(<EM>var1, value1, descript1, var2,
value2, descript2,...</EM>)</B></CODE>: Sends a set of
checkboxes. Each var/value/descript pair corresponds to a
checkbox. The var parameter is the variable to fill (as a string), the
value parameter is its default value, and the descript field is the
text places to the right of the checkbox. <P>

<LI> <B><CODE>HTML_STRING(<EM>descript, size, var,
value</EM>)</B></CODE>: Sends a string field. The first parameter is a
description of the data to be entered. It will appear above the
field. The second parameter is the size, in characters, of the data
field. The var field is the variable to fill (again, as a string), and
the last parameter is the default value. <P>

<LI> <B><CODE>HTML_SAVE_DATA(<EM>var</EM>)</B></CODE>: Typically, the
<CODE>HTML_</CODE> functions above will use variable names of the form
<CODE>VAR{'var'}</CODE>, where <CODE>VAR</CODE> is an associative
array, and <CODE>var</CODE> is the index into it. Once a tool has been
executed, the parameters used to run it may be saved. To save the
parameters for a tool named VAR, enter
<CODE>HTML_SAVE_DAT("VAR")</CODE>. This will generated a parameter
file (Perl assignment statements) in the directory
<CODE>/parameters</CODE> for that tool. <P>

</UL>
EOF
;


&HTML_MISC_FOOTER();
