&HTML_MISC_HEADER("Developer's Guide");

print CLIENT <<EOF

<A NAME="overview"><H1>Merlin Overview</H1>

This document is broken into two sections. The first discusses the
internal design of Merlin. The second discusses its directory
structure.

<A NAME="design"><H2>Design</H2>

<IMG SRC="diagram.gif"> <P>

Merlin can be thought of as a private WWW server. As show above, an
HTTP server receives messages from an HTTP client. These messages are
processed and appropriate actions are taken. The server only accepts
requests from a client on the local host who knows the correct port
number and the correct password (Merlin automatically forks off
such a client upon starting up).<P>

The server manages all tools. All tool-specific code is stored in the
same directory hierarchy. When starting, Merlin first checks to see
what packages are supported, and of those, which packages have been
set up in Merlin. <P>

So, to provide support for a new package, make a new subdirectory for
it and write the appropriate code. Merlin will automatically detect
the package upon startup (via the merlin_config file). Package support
for new tools may be distributed separately from Merlin and added into
the package hierarchy.

Two libraries are provided which offer common package related
functions. See the section <A HREF="#library">Tools Library</A> for
more information.

<A NAME="hierarchy"><H2> Directory Hierarchy</H2>

The following directories make up the HTTP server (<CODE>~</CODE>
stands for your Merlin root directory): <UL>

<LI><CODE><B>~/:</B></CODE> The HTTP server is stored here
(<CODE>merlin</CODE>), in addition to two configuration files
(<CODE>setup.pl</CODE> and <CODE>config.pl</CODE>) and misc. other
junk.<P>

<LI><CODE><B>~/perl:</B></CODE> Two libraries used for simplifying the
tools support are provided here (<CODE>tools.pl and
</CODE><CODE>html.pl</CODE>).<P>

<LI><CODE><B>~/src:</B></CODE> Only one program is stored here,
<CODE>headers.c</CODE>. This avoids the use of the Perl program h2ph,
which has a tendency to generate broken .ph files. Any declarations
needed by Merlin are generated by this program.<P>

<LI><CODE><B>~/bin:</B></CODE> The binary version of
<CODE>headers</CODE> is stored here.<P>

<LI><CODE><B>~/html:</B></CODE> All HTML-producing Perl code not
specific to a tool is stored at this location. Each time Merlin is
started, or the package configuration is changed, the
<CODE>merlin.pl</CODE> generates
<CODE>merlin.html</CODE>, the initial Merlin home page.<P>

<LI><CODE><B>~/html/status:</B></CODE> Job management code is stored
here.<P>

<LI><CODE><B>~/html/reports:</B></CODE> Report browser code is stored
here.<P>

<LI><CODE><B>~/html/manual:</B></CODE> All documentation is stored
here.<P>

</UL>

<UL>
<LI><CODE><B>~/pkg:</B></CODE> Each package is stored in a separate
subdirectory under here. Each package can contain one or more tools.<P>

</UL>

Output and process control files are stored here:
<UL>
<LI><CODE><B>~/results:</B></CODE>The output for each job is
stored here. The filename structure is identical for the results,
status, and errors directories. <P>

<LI><CODE><B>~/status:</B></CODE> Each job will store its group
process ID in this directory. <P>

<LI><CODE><B>~/errors:</B></CODE> Each job routes its
<CODE>stderr</CODE> to a file in this directory. When viewing the
job's report, any error messages are inserted after the report. <P>

<LI><CODE><B>~/parameters:</B></CODE> The last parameters used to
execute each tool are stored here.
</UL>

<P><BR>

<A NAME="server"><H1>The HTTP Server</H1>

Merlin's central nervous system is a Perl-based HTTP (World Wide Web)
Server. While it works in a manner similiar to other servers, several
aspects of it are worth discussing:

<UL>

<LI> Several security measures prevent the server from accepting
requests from anyone except the Netscape client Merlin started. See
the <A HREF="intro.html.pl">Introduction</A> for more details. <P>

<LI> A request containing the string "<CODE>.pl</CODE>" will be
considered a Perl script, and executed accordingly (see <A
HREF="#makelink">Making a Link</A> for more details). <P>

<LI> All information written to the CLIENT filehandle will be sent
back to the Netscape client. <P>

</UL>


<A NAME="makelink"><H2>Making a Link</H2>

Here is an example of a complete Merlin link,
<CODE>execute.pl</CODE> from the COPS package support: <P>

<CODE>
\&HTML_MISC_HEADER("Computer Oracle Password&ltBR&gtand Security System");<BR>
print CLIENT &lt&ltEOF<BR>
&ltFORM METHOD=POST ACTION="cops_run.pl"&gt<BR>
&ltINPUT TYPE="submit" VALUE= "Start the Scan "&gt<BR>
&lt/FORM&gt<BR>
EOF<BR>
;<BR>
\&HTML_MISC_FOOTER();<P>
</CODE>

This is a very simple HTML form. It contains a header and a footer
(HTML code produced by HTML_MISC_HEADER and HTML_MISC_FOOTER,
respectively), and a submit button. All HTML code to be sent to the
client is sent to the CLIENT filehandle. <P>

When the button marked "Start the Scan" is selected, the script
<CODE>cops_run.pl</CODE> will be executed. <P>

<A NAME="parameters"><H2>Parameter Passing</H2>

Arguments to a script may be passed by putting them at the end of
a link name. The script can find the arguments in the variable
<CODE>\$html_script_args</CODE>. For example, for the link
<CODE>&lt;A HREF="myprog.pl,23"&gt;</CODE>, the string
<CODE>\$html_script_args</CODE> will have the value "<CODE>23</CODE>". <P>

Data may also be passed to scripts using the POST mechanism. HTML
forms utilize the POST mechanism. The NAME field of a form tag should
contain the name of the variable pass. For example, here is a simple
HTML form: <P>

<CODE>
&lt;FORM METHOD=POST ACTION="process.pl"&gt;<BR>
&lt;INPUT SIZE=30 NAME=foobar VALUE="test field"&gt;<BR>
&lt;INPUT TYPE="submit" VALUE="Start the Test"&gt;<P>
</CODE>

When the Perl script <CODE>process.pl</CODE> is executed, the
variable \$foobar will contain whatever was typed by the user into
this string field. Any type of scalar variable is possible, including
an array element. <P>

Note that, for the most part, raw HTML form code will not be
necessary. Merlin contains a library for automating this task. See the
<A HREF="#library">Tools Library</A> for more information. <P>

<A NAME="addtools"><H1>Adding Package Support</H1>


<A NAME="basics"><H2>The Basics</H2>

This section discusses how package support is implemented. Hopefully,
enough information is provided here to allow others to easily add
support for their favorite tools. The best way to understand how to
extend Merlin, however, is Trial-By-Example; browse through the
package support that has already been written.<P>

Adding support for a new package can be very simple, or very complex,
depending on your needs and the nature of the tool. Two libraries,
discussed in the next section, provide a set of functions to make this
development easier. <P>

Each package is made up of one or more tools. Each package as well as
each tool is refered to by a three letter code: the spi (SPI) package
includes bat, psi, cdt, and cql; the tgr (Tiger) package includes just
tgr.  The package code must be unique among all packages, and the tool
code must be unique among all tools, since they are used as
identifiers throughout Merlin. <P>

As far as Merlin is concerned, only three files are required for a new
tool: <CODE>merlin_config</CODE>, <CODE>execute.pl</CODE>, and
<CODE>read_report.pl.</CODE> All other file name are up to the
developer, although they must end in <CODE>.pl</CODE> if they are
written in Perl. Below is a closer look at the files required:

<H3>merlin_config</H3>

This is a data file which contains the name of the package, the code
for the package, and where the package can be found. It also contains
one or more tool descriptions (and their corresponding codes). The
formatting is as follows: <P>

<CODE>
[package code]:[directory]:[package title]<BR>
[tool1 code]:[tool1 title]<BR>
[tool2 code]:[tool2 title]<P>
</CODE>

The <CODE>merlin_config</CODE> file for SPI looks like this: <P>

<CODE>spi::Security Profile Inspector<BR>
bat:Binary Authentication Tool<BR>
qsp:Quick System Profile<BR>
psi:Password Security Inspector<BR>
cdt:Change Detector Tool<P>
</CODE>

Note that the directory location is blank; it will be filled in by the
Configuration link. <P>

<H3>execute.pl</H3>

This is the link to be offered from Merlin's Table of Contents. In the
case of TAMU Tiger, a form is offered, where the user can select which
checks to perform. For SPI, a sub-menu of tools offered by the
packages is provided.

<H3>read_report.pl</H3>

When a report is selected from the Report Browser, this program is
executed. It should read the report, and return to the CLIENT the
HTML-enhanced version. How this is done depends on the tool. SPI, for
example, uses the <CODE>rg</CODE> program included with SPI, but uses
a different template file. The other tools actually parse the report
file. The argument passed in \$html_script_args is the file name to
display (see <A HREF="server.html.pl">The HTTP Server</A> for more
details on argument passing).

<A NAME="library"><H2>Development Libraries</H2>

The following functions are offered to all packages, in the file
<CODE>perl/tools.pl</CODE>:

<UL> <LI> <B><CODE>NTOM(<EM>num</EM>)</B></CODE>: Convert a numeric
value between 1 and 12 to its month equivalent. <P>

<LI> <B><CODE>TOOL_PARSE_FILE(<EM>filename</EM>)</B></CODE>: Take as
input a valid file and return an array containing the date, the time,
the host and the tool. <P>

The filename is formatted as follows:
<PRE>
[tool].[date].[time].[host]
</PRE>

where<P>

[tool] is a three character description of the tool executed. It
should be defined in the merlin_config file for that tool. <EM>Note
that this three character description should be unique among all
packages.</EM><P>

[date] is a 8 digit date, such as 19950713 for July 13, 1995.<P>

[time] is a 6 digit time, such as 165150 for 4:51 pm and 50 seconds.<P>

[host] is an arbitarly long host name (for now, always the local host).<P>


<LI> <B><CODE>TOOL_CREATE_FILE(<EM>tool</EM>)</B></CODE>: Create a
tool filename, formatted as described for TOOL_PARSE_FILE(). It takes
as a parameter a three character tool descriptor. <P>

<LI> <B><CODE>TOOL_RUN_STATUS(<EM>filename, pgid</EM>)</B></CODE>:
Take as input a process group ID, followed by filename (as created
with TOOL_CREATE_FILE()). It then writes the process group ID into the
status directory, under that filename. <P>

</UL>

Output to the HTTP Client should be sent to the file descriptor
CLIENT. To simplify the generation of HTML code, the following
functions are provided, in the file <CODE>perl/html.pl</CODE>:

<UL>

<LI> <B><CODE>HTML_MISC_HEADER(<EM>title</EM>)</B></CODE>: Sends the
standard Merlin HTML page header. This function is used by virtually
all pages, excluding the home page and the reports. <P>


<LI> <B><CODE>HTML_MISC_FOOTER()</B></CODE>: Sends the standard Merlin
HTML page footer. This function is used by virtually all pages,
excluding the home page and the reports. <P>


<LI> <B><CODE>HTML_REPORT_HEADER(<EM>file, title</EM>)</B></CODE>:
Sends the Merlin HTML page header used for reports. <P>

<LI> <B><CODE>HTML_RADIOS(<EM>variable, value, descript1,
[descript2],...</EM>)</B></CODE>: Sends a set of radio buttons. The
first parameter is the name of the variable to fill (sent as a
string). The second parameter is the default value. Each subsequent
descript parameter is a description for each button in the set. <P>

<LI> <B><CODE>HTML_CHECKBOXES(<EM>var1, value1, descript1, [var2,
value2, descript2],...</EM>)</B></CODE>: Sends a set of
checkboxes. Each var/value/descript pair corresponds to a
checkbox. The var parameter is the variable to fill (as a string), the
value parameter is its default value, and the descript field is the
text places to the right of the checkbox. <P>

<LI> <B><CODE>HTML_STRING(<EM>descript, size, var,
value</EM>)</B></CODE>: Sends a string field. The first parameter is a
description of the data to be entered. It will appear above the
field. The second parameter is the size, in characters, of the data
field. The var field is the variable to fill (again, as a string), and
the last parameter is the default value. <P>

<LI> <B><CODE>HTML_SAVE_DATA(<EM>var</EM>)</B></CODE>: Typically, the
<CODE>HTML_</CODE> functions above will use variable names of the form
<CODE>VAR{'var'}</CODE>, where <CODE>VAR</CODE> is an associative
array, and <CODE>var</CODE> is the index into it. Once a tool has been
executed, the parameters used to run it may be saved. To save the
parameters for a tool named VAR, enter
<CODE>HTML_SAVE_DAT("VAR")</CODE>. This will generated a parameter
file (Perl assignment statements) in the directory
<CODE>/parameters</CODE> for that tool. <P>

</UL>

 <P>
EOF
;


&HTML_MISC_FOOTER();
